﻿Imports System.Windows.Forms.DataVisualization.Charting

Public Class iDFT_XO

    Dim Initialized As Integer = 0

    Dim FS As Integer 'sampling frequency in Hz
    Dim FIR_length As Integer = 1000 'longest FIR that's supported
    Dim FFT_length As Integer = 65536 'longest FFT that's supported

    Dim order As Double

    Dim a0, a1, a2, a3, a4 As Double 'Nuttall, Blackman, Harris, etc ... windows coefficients 

    Dim Fcut As Double
    Dim n0 As Double 'numerator Gain coefficients
    Dim d0, d2, d4, d6, d8, d10, d12 As Double 'denominator Gain coefficients
    Dim Fscale_factor As Double
    Dim GAIN(FIR_length) As Double
    Dim PHASE(FIR_length) As Double
    Dim GAIN_LPF_TARGET(FIR_length) As Double

    Dim F As Double 'current frequency in Hz
    Dim Fr As Double 'relative current frequency (equals 1 when Fcut)

    'create FIR buffers with maximal FIR length

    Dim FIR(FIR_length) As Double
    Dim WIN(FIR_length) As Double
    Dim FIRLPF(FIR_length) As Double
    Dim FIRHPF(FIR_length) As Double
    Dim FIR_norm(FIR_length) As Double
    Dim FIRLPF_norm(FIR_length) As Double
    Dim FIRHPF_norm(FIR_length) As Double

    'create signal-related buffers
    Dim BUF_length As Integer = FIR_length + FFT_length
    Dim Signal_window(BUF_length) As Double 'Window
    Dim Signal_noise(BUF_length) As Double 'Signal 
    Dim Signal_LPF(BUF_length) As Double 'LPF signal out
    Dim Signal_HPF(BUF_length) As Double 'HPF signal out
    Dim FIR_in(BUF_length) As Double
    Dim FIRLPF_out(BUF_length) As Double
    Dim FIRHPF_out(BUF_length) As Double

    'create FFT-related buffers
    Dim FFT_in(FFT_length) As Double
    Dim FFT_out_module(FFT_length) As Double
    Dim FFT_out_phase(FFT_length) As Double
    Dim Signal_module(FFT_length) As Double
    Dim Signal_LPF_module(FFT_length) As Double
    Dim Signal_HPF_module(FFT_length) As Double

    Dim REX(FFT_length) As Double 'holds the real part of the frequency domain
    Dim IMX(FFT_length) As Double 'holds the imaginary part of the frequency domain
    Dim XX(FFT_length) As Double 'holds the time domain signal

    Dim N, NM1, NM1D2 As Integer
    Dim MID, NAM As Integer

    Dim x As Integer
    Dim sum As Double

    Dim ACC As Double 'accumulator in FIR
    Dim val As Double
    Dim arg As Double

    Dim path As String = My.Computer.FileSystem.SpecialDirectories.MyDocuments
    Dim file As String = "iDFT_XO.txt"
    Dim pathfile As String


    Private Sub iDFTlab_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        Me.Label_path.Text = path
        Me.Label_file.Text = "FIR coeffs are written into " + file

        Me.FS_List.SelectedIndex = 1 '48000 Hz

        Me.FIR_length_List.SelectedIndex = 10 '101-tap
        FIR_length = Me.FIR_length_List.Text
        N = FIR_length
        NM1 = N - 1
        NM1D2 = (N - 1) / 2
        MID = (FIR_length + 1) / 2 'median index 
        NAM = FIR_length - MID 'points after median = points before median

        Me.BodeSigGen_List.Enabled = False
        Me.BodeSigGen_List.SelectedIndex = 0 'White Source Generator

        Me.BodeWindow_List.Enabled = False
        Me.BodeWindow_List.SelectedIndex = 1 'Triangle Windowing for FFT Bode plot

        Me.Fmin_Combo.SelectedIndex = 2 '100 Hz
        Me.Fmax_Combo.SelectedIndex = 2 '20000 Hz
        Me.dBmax_Combo.SelectedIndex = 1 '-10 dB
        Me.dBmin_Combo.SelectedIndex = 4 '-60 dB

        Me.FFT_length_List.SelectedIndex = 10 '16384
        FFT_length = Me.FFT_length_List.Text

        Me.Lowpass_List.SelectedIndex = 6 'Symmetric
        Me.F_Label.Text = "F-6dB(Hz)"
        Me.Order_List.SelectedIndex = 3 '4th-order
        Me.Double_Box.Checked = False 'when enabled, serves emulating Linkwitz-Riley in Butterwoth modality
        Me.Window_List.SelectedIndex = 3 'Cosine Windowing

        Fcut = 3400 'initialize Fcut parameter
        Me.Fcut_Box.Text = Fcut 'load Fcut parameter in Box

        Initialized = 1

        ComputeALL()

    End Sub

    Private Sub Go_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Apply.Click

        ComputeALL()

    End Sub

    Private Sub ComputeALL()

        If Initialized = 1 Then

            Fcut = Me.Fcut_Box.Text 'read Fcut parameter

            N = FIR_length
            NM1 = N - 1
            NM1D2 = (N - 1) / 2
            MID = (FIR_length + 1) / 2 'median index 
            NAM = FIR_length - MID 'points after median = points before median

            For i As Integer = 0 To N 'clear all FIR coefficients
                FIR(i) = 0
                FIR_norm(i) = 0
                WIN(i) = 0
                FIRLPF(i) = 0
                FIRLPF_norm(i) = 0
                FIRHPF_norm(i) = 0
            Next

            If Me.Lowpass_List.SelectedIndex = 0 Then 'Brickwall -40dB
                Me.F_Label.Text = "F-3dB(Hz)"
                Me.Order_List.Enabled = False
                Me.Double_Box.Enabled = False
                For i = 0 To N
                    F = FS * (i / N)
                    If F < Fcut Then GAIN(i) = 1 Else GAIN(i) = 0.01
                    PHASE(i) = 2 * Math.PI * (MID * i / N) 'take FIR delay in account for the phase 
                Next
            End If

            If Me.Lowpass_List.SelectedIndex = 1 Then 'Brickwall -60dB
                Me.F_Label.Text = "F-3dB(Hz)"
                Me.Order_List.Enabled = False
                Me.Double_Box.Enabled = False
                For i = 0 To N
                    F = FS * (i / N)
                    If F < Fcut Then GAIN(i) = 1 Else GAIN(i) = 0.001
                    PHASE(i) = 2 * Math.PI * (MID * i / N) 'take FIR delay in account for the phase 
                Next
            End If

            If Me.Lowpass_List.SelectedIndex = 2 Then 'Brickwall -80dB
                Me.F_Label.Text = "F-3dB(Hz)"
                Me.Order_List.Enabled = False
                Me.Double_Box.Enabled = False
                For i = 0 To N
                    F = FS * (i / N)
                    If F < Fcut Then GAIN(i) = 1 Else GAIN(i) = 0.0001
                    PHASE(i) = 2 * Math.PI * (MID * i / N) 'take FIR delay in account for the phase 
                Next
            End If

            If Me.Lowpass_List.SelectedIndex = 3 Then 'Brickwall -100dB
                Me.F_Label.Text = "F-3dB(Hz)"
                Me.Order_List.Enabled = False
                Me.Double_Box.Enabled = False
                For i = 0 To N
                    F = FS * (i / N)
                    If F < Fcut Then GAIN(i) = 1 Else GAIN(i) = 0.00001
                    PHASE(i) = 2 * Math.PI * (MID * i / N) 'take FIR delay in account for the phase 
                Next
            End If

            If Me.Lowpass_List.SelectedIndex = 4 Then 'Brickwall -120dB
                Me.F_Label.Text = "F-3dB(Hz)"
                Me.Order_List.Enabled = False
                Me.Double_Box.Enabled = False
                For i = 0 To N
                    F = FS * (i / N)
                    If F < Fcut Then GAIN(i) = 1 Else GAIN(i) = 0.000001
                    PHASE(i) = 2 * Math.PI * (MID * i / N) 'take FIR delay in account for the phase 
                Next
            End If

            If Me.Lowpass_List.SelectedIndex = 5 Then 'Asymptot
                Me.F_Label.Text = "Fcut(Hz)"
                Me.Order_List.Enabled = True
                Me.Double_Box.Enabled = False
                For i = 0 To N
                    F = FS * (i / N)
                    If F < Fcut Then GAIN(i) = 1 Else GAIN(i) = (Fcut / F) ^ (Me.Order_List.SelectedIndex + 1)
                    PHASE(i) = 2 * Math.PI * (MID * i / N) 'take FIR delay in account for the phase 
                Next
            End If

            If Me.Lowpass_List.SelectedIndex = 6 Then 'Symmetric
                Me.F_Label.Text = "F-6dB(Hz)"
                Me.Order_List.Enabled = True
                order = Me.Order_List.SelectedIndex + 1
                Me.Double_Box.Enabled = False

                For i = 0 To N
                    F = FS * (i / N)
                    If F < Fcut Then GAIN(i) = 1 - (((F / Fcut) ^ order) / 2.0)
                    If F = Fcut Then GAIN(i) = 0.5
                    If F > Fcut Then GAIN(i) = (((Fcut / F) ^ order) / 2.0)
                    PHASE(i) = 2 * Math.PI * (MID * i / N) 'take FIR delay in account for the phase 
                Next

            End If

            If Me.Lowpass_List.SelectedIndex = 7 Then 'Butterworth

                Me.F_Label.Text = "F-3dB(Hz)"
                Me.Order_List.Enabled = True
                Me.Double_Box.Enabled = True

                If Me.Order_List.SelectedIndex = 0 Then
                    n0 = 1
                    d0 = 1
                    d2 = 1
                    d4 = 0
                    d6 = 0
                    d8 = 0
                    d10 = 0
                    d12 = 0
                End If

                If Me.Order_List.SelectedIndex = 1 Then
                    n0 = 1
                    d0 = 1
                    d2 = 0
                    d4 = 1
                    d6 = 0
                    d8 = 0
                    d10 = 0
                    d12 = 0
                End If

                If Me.Order_List.SelectedIndex = 2 Then
                    n0 = 1
                    d0 = 1
                    d2 = 0
                    d4 = 0
                    d6 = 1
                    d8 = 0
                    d10 = 0
                    d12 = 0
                End If

                If Me.Order_List.SelectedIndex = 3 Then
                    n0 = 1
                    d0 = 1
                    d2 = 0
                    d4 = 0
                    d6 = 0
                    d8 = 1
                    d10 = 0
                    d12 = 0
                End If

                If Me.Order_List.SelectedIndex = 4 Then
                    n0 = 1
                    d0 = 1
                    d2 = 0
                    d4 = 0
                    d6 = 0
                    d8 = 0
                    d10 = 1
                    d12 = 0
                End If

                If Me.Order_List.SelectedIndex = 5 Then
                    n0 = 1
                    d0 = 1
                    d2 = 0
                    d4 = 0
                    d6 = 0
                    d8 = 0
                    d10 = 0
                    d12 = 1
                End If

                For i = 0 To N
                    F = FS * (i / N)
                    Fr = F / Fcut
                    GAIN(i) = n0 / ((d0 + d2 * Fr ^ 2 + d4 * Fr ^ 4 + d6 * Fr ^ 6 + d8 * Fr ^ 8 + d10 * Fr ^ 10 + d12 * Fr ^ 12)) ^ 0.5
                    If Me.Double_Box.Checked = True Then GAIN(i) = GAIN(i) * GAIN(i)
                    PHASE(i) = 2 * Math.PI * (MID * i / N) 'take FIR delay in account for the phase 
                Next

            End If

            If Me.Lowpass_List.SelectedIndex = 8 Then  'Bessel

                Me.F_Label.Text = "F-3dB(Hz)"
                Me.Order_List.Enabled = True
                Me.Double_Box.Enabled = False

                If Me.Order_List.SelectedIndex = 0 Then
                    Fscale_factor = 1
                    n0 = 1
                    d0 = 1
                    d2 = 1
                    d4 = 0
                    d6 = 0
                    d8 = 0
                    d10 = 0
                    d12 = 0
                End If

                If Me.Order_List.SelectedIndex = 1 Then
                    Fscale_factor = 1.362
                    n0 = 3
                    d0 = 9
                    d2 = 3
                    d4 = 1
                    d6 = 0
                    d8 = 0
                    d10 = 0
                    d12 = 0
                End If

                If Me.Order_List.SelectedIndex = 2 Then
                    Fscale_factor = 1.756
                    n0 = 15
                    d0 = 225
                    d2 = 45
                    d4 = 6
                    d6 = 1
                    d8 = 0
                    d10 = 0
                    d12 = 0
                End If

                If Me.Order_List.SelectedIndex = 3 Then
                    Fscale_factor = 2.114
                    n0 = 105
                    d0 = 11025
                    d2 = 1575
                    d4 = 135
                    d6 = 10
                    d8 = 1
                    d10 = 0
                    d12 = 0
                End If

                If Me.Order_List.SelectedIndex = 4 Then
                    Fscale_factor = 2.427
                    n0 = 945
                    d0 = 893025
                    d2 = 99225
                    d4 = 6300
                    d6 = 315
                    d8 = 15
                    d10 = 1
                    d12 = 0
                End If

                If Me.Order_List.SelectedIndex = 5 Then
                    Fscale_factor = 2.703
                    n0 = 10395
                    d0 = 108056025
                    d2 = 9823275
                    d4 = 496125
                    d6 = 18900
                    d8 = 630
                    d10 = 21
                    d12 = 1
                End If

                For i = 0 To N
                    F = FS * (i / N)
                    Fr = F / Fcut
                    Fr = Fr * Fscale_factor
                    GAIN(i) = n0 / ((d0 + d2 * Fr ^ 2 + d4 * Fr ^ 4 + d6 * Fr ^ 6 + d8 * Fr ^ 8 + d10 * Fr ^ 10 + d12 * Fr ^ 12)) ^ 0.5
                    PHASE(i) = 2 * Math.PI * (MID * i / N) 'take FIR delay in account for the phase 
                Next

            End If

            For i As Integer = 0 To NM1D2 'store GAIN_LPF_TARGET
                GAIN_LPF_TARGET(i) = GAIN(i)
            Next

            For i As Integer = 0 To NM1D2 'load REX & IMX buffers with data
                REX(i) = GAIN(i) * Math.Cos(PHASE(i))
                IMX(i) = GAIN(i) * Math.Sin(PHASE(i))
            Next

            do_iDFT() 'input is REX(i) & IMX(i), output is XX(i)

            For i As Integer = 0 To N
                FIR(i + 1) = XX(i) 'lowpass FIR 
            Next

            If Me.Window_List.SelectedIndex = 0 Then 'Rectangle
                For i As Integer = 1 To N
                    WIN(i) = 1
                Next
            End If

            If Me.Window_List.SelectedIndex = 1 Then 'Triangle
                WIN(MID) = 1
                For i As Integer = 1 To NAM
                    val = 1 - (i / MID)
                    WIN(MID + i) = val
                    WIN(MID - i) = val
                Next
            End If

            If Me.Window_List.SelectedIndex = 2 Then 'Bartlett-Hann
                a0 = 0.62
                a1 = -0.48
                a2 = 0.38
                For i As Integer = 0 To N
                    val = a0
                    val = val + a1 * Math.Abs((i / NM1) - 0.5)
                    val = val + a2 * Math.Cos(2 * Math.PI * ((i / NM1) - 0.5))
                    WIN(i + 1) = val
                Next
            End If

            If Me.Window_List.SelectedIndex = 3 Then 'Cosine
                For i As Integer = 0 To N
                    val = Math.Sin((Math.PI * i) / NM1)
                    WIN(i + 1) = val
                Next
            End If

            If Me.Window_List.SelectedIndex = 4 Then 'Hann
                a0 = 0.5
                a1 = -0.5
                a2 = 0.0
                a3 = 0.0
                a4 = 0.0
                For i As Integer = 0 To N
                    val = a0
                    val = val + a1 * Math.Cos((2 * Math.PI * i) / (NM1))
                    val = val + a2 * Math.Cos((4 * Math.PI * i) / (NM1))
                    val = val + a3 * Math.Cos((6 * Math.PI * i) / (NM1))
                    val = val + a4 * Math.Cos((8 * Math.PI * i) / (NM1))
                    WIN(i + 1) = val
                Next
            End If

            If Me.Window_List.SelectedIndex = 5 Then 'Hamming
                a0 = 0.54
                a1 = -0.46
                a2 = 0.0
                a3 = 0.0
                a4 = 0.0
                For i As Integer = 0 To N
                    val = a0
                    val = val + a1 * Math.Cos((2 * Math.PI * i) / (NM1))
                    val = val + a2 * Math.Cos((4 * Math.PI * i) / (NM1))
                    val = val + a3 * Math.Cos((6 * Math.PI * i) / (NM1))
                    val = val + a4 * Math.Cos((8 * Math.PI * i) / (NM1))
                    WIN(i + 1) = val
                Next
            End If

            If Me.Window_List.SelectedIndex = 6 Then 'Blackman
                a0 = 0.42659
                a1 = -0.49656
                a2 = 0.076849
                a3 = 0.0
                a4 = 0.0
                For i As Integer = 0 To N
                    val = a0
                    val = val + a1 * Math.Cos((2 * Math.PI * i) / (NM1))
                    val = val + a2 * Math.Cos((4 * Math.PI * i) / (NM1))
                    val = val + a3 * Math.Cos((6 * Math.PI * i) / (NM1))
                    val = val + a4 * Math.Cos((8 * Math.PI * i) / (NM1))
                    WIN(i + 1) = val
                Next
            End If

            If Me.Window_List.SelectedIndex = 7 Then 'Nuttall
                a0 = 0.356768
                a1 = -0.487396
                a2 = 0.144232
                a3 = -0.012604
                a4 = 0.0
                For i As Integer = 0 To N
                    val = a0
                    val = val + a1 * Math.Cos((2 * Math.PI * i) / (NM1))
                    val = val + a2 * Math.Cos((4 * Math.PI * i) / (NM1))
                    val = val + a3 * Math.Cos((6 * Math.PI * i) / (NM1))
                    val = val + a4 * Math.Cos((8 * Math.PI * i) / (NM1))
                    WIN(i + 1) = val
                Next
            End If

            If Me.Window_List.SelectedIndex = 8 Then 'Blackman-Harris
                a0 = 0.35875
                a1 = -0.48829
                a2 = 0.14128
                a3 = -0.01168
                a4 = 0.0
                For i As Integer = 0 To N
                    val = a0
                    val = val + a1 * Math.Cos((2 * Math.PI * i) / (NM1))
                    val = val + a2 * Math.Cos((4 * Math.PI * i) / (NM1))
                    val = val + a3 * Math.Cos((6 * Math.PI * i) / (NM1))
                    val = val + a4 * Math.Cos((8 * Math.PI * i) / (NM1))
                    WIN(i + 1) = val
                Next
            End If

            If Me.Window_List.SelectedIndex = 9 Then 'Blackmann-Nuttall
                a0 = 0.3635819
                a1 = -0.4891775
                a2 = 0.1365995
                a3 = -0.0106411
                a4 = 0.0
                For i As Integer = 0 To N
                    val = a0
                    val = val + a1 * Math.Cos((2 * Math.PI * i) / (NM1))
                    val = val + a2 * Math.Cos((4 * Math.PI * i) / (NM1))
                    val = val + a3 * Math.Cos((6 * Math.PI * i) / (NM1))
                    val = val + a4 * Math.Cos((8 * Math.PI * i) / (NM1))
                    WIN(i + 1) = val
                Next
            End If

            If Me.Window_List.SelectedIndex = 10 Then 'Flat top
                a0 = 1.0
                a1 = -1.93
                a2 = 1.29
                a3 = -0.388
                a4 = 0.032
                For i As Integer = 0 To N
                    val = a0
                    val = val + a1 * Math.Cos((2 * Math.PI * i) / (NM1))
                    val = val + a2 * Math.Cos((4 * Math.PI * i) / (NM1))
                    val = val + a3 * Math.Cos((6 * Math.PI * i) / (NM1))
                    val = val + a4 * Math.Cos((8 * Math.PI * i) / (NM1))
                    WIN(i + 1) = val
                Next
            End If

            For i As Integer = 0 To N
                FIRLPF(i) = FIR(i) * WIN(i)
            Next

            sum = 0
            For i As Integer = 1 To N 'compute the sum of all coefficients
                sum = sum + FIRLPF(i)
            Next
            For i As Integer = 1 To N 'normalize coefficients (make sum equal to 1)
                FIRLPF_norm(i) = (FIRLPF(i)) / sum
            Next

            'export the LPF coefficient list to a text file

            pathfile = System.IO.Path.Combine(My.Computer.FileSystem.SpecialDirectories.MyDocuments, file)
            Dim objWriter As New System.IO.StreamWriter(pathfile)
            For i As Integer = 1 To N
                objWriter.WriteLine(FIRLPF_norm(i))
            Next
            objWriter.Close()

            ' Start FIR plot

            Me.FIRPlot.Series.Clear()

            With Me.FIRPlot
                With .Series.Add("FIR_LPF")
                    For i As Integer = 1 To N
                        .Points.AddXY(i - MID, 10 * FIRLPF_norm(i))
                    Next
                End With
            End With
            Me.FIRPlot.Series("FIR_LPF").BorderWidth = 1
            Me.FIRPlot.Series("FIR_LPF").Color = Color.Blue
            Me.FIRPlot.Series("FIR_LPF").ChartType = DataVisualization.Charting.SeriesChartType.Line

            ' FIR LPF is now plotted

            'compute FIR HPF
            For i As Integer = 0 To N 'compute complementary FIR (= highpass)
                FIRHPF_norm(i) = -(FIRLPF_norm(i))
            Next
            FIRHPF_norm(MID) = 1 - FIRLPF_norm(MID)

            With Me.FIRPlot
                With .Series.Add("FIR_HPF")
                    For i As Integer = 1 To N
                        .Points.AddXY(i - MID, 10 * FIRHPF_norm(i))
                    Next
                End With
            End With
            Me.FIRPlot.Series("FIR_HPF").BorderWidth = 1
            Me.FIRPlot.Series("FIR_HPF").Color = Color.Red
            Me.FIRPlot.Series("FIR_HPF").ChartType = DataVisualization.Charting.SeriesChartType.Line

            ' FIR HPF is now plotted

            Signal_generate() 'generate the source signal used for the Bode Plot (should be white noise)
            Signal_window_generate() 'generate weighting window

            x = 1
            For i As Integer = MID To (FFT_length + MID - 1) ' transfer windowed signal into FFT routine
                FFT_in(x) = Signal_window(i) * Signal_noise(i) 'multiply signal by window
                x = x + 1
            Next
            FFT() 'execute FFT routine on input signal
            For i As Integer = 1 To (FFT_length) 'save input signal module
                Signal_module(i) = FFT_out_module(i)
            Next

            For i As Integer = 0 To (BUF_length) 'transfer source signal to the FIR filter
                FIR_in(i) = Signal_noise(i)
            Next

            do_FIR() 'execute FIRLPF filter and FIRHPF on input signal

            For i As Integer = 0 To (BUF_length) 'save FIRLPF filtered signal
                Signal_LPF(i) = FIRLPF_out(i)
            Next
            x = 1
            For i As Integer = MID To (FFT_length + MID - 1) ' transfer LPF filtered windowed signal into FFT routine
                FFT_in(x) = Signal_window(i) * Signal_LPF(i)
                x = x + 1
            Next
            FFT() 'execute FFT routine on FIRLPF filtered signal
            For i As Integer = 0 To (FFT_length) 'save FIRLPF filtered signal module
                Signal_LPF_module(i) = FFT_out_module(i)
            Next

            For i As Integer = 0 To (BUF_length) 'save FIRHPF filtered signal
                Signal_HPF(i) = FIRHPF_out(i)
            Next
            x = 1
            For i As Integer = MID To (FFT_length + MID - 1) ' transfer HPF filtered windowed signal into FFT routine
                FFT_in(x) = Signal_window(i) * Signal_HPF(i)
                x = x + 1
            Next
            FFT() 'execute FFT routine on FIRHPF filtered signal
            For i As Integer = 0 To (FFT_length) 'save FIRHPF filtered signal module
                Signal_HPF_module(i) = FFT_out_module(i)
            Next

            Bode_plot()

        End If

    End Sub

    Private Sub Signal_window_generate()

        If Me.BodeWindow_List.SelectedIndex = 0 Then 'Rectangle Window
            For i As Integer = 0 To (BUF_length)
                Signal_window(i) = 1
            Next
        End If

        If Me.BodeWindow_List.SelectedIndex = 1 Then 'Triangle Window
            x = 1
            For i As Integer = MID To (MID + (FFT_length / 2))
                Signal_window(i) = x / FFT_length
                x = x + 1
            Next
            For i As Integer = (MID + (FFT_length / 2) + 1) To (FFT_length + MID - 1)
                Signal_window(i) = 1 - (x / FFT_length)
                x = x + 1
            Next
        End If

    End Sub


    Private Sub do_FIR()

        FFT_Bar.Value = 0

        x = MID
        For i As Integer = 0 To (FFT_length - 1)
            ACC = 0
            For j = 1 To N
                ACC = ACC + (FIR_in(i + j) * FIRLPF_norm(j))
            Next
            FIRLPF_out(x) = ACC
            x = x + 1
            FIR_Bar.Value = (i / FFT_length) * 100
        Next

        x = MID
        For i As Integer = 0 To (FFT_length - 1)
            ACC = 0
            For j = 1 To N
                ACC = ACC + (FIR_in(i + j) * FIRHPF_norm(j))
            Next
            FIRHPF_out(x) = ACC
            x = x + 1
            FIR_Bar.Value = (i / FFT_length) * 100
        Next

    End Sub

    Public Sub do_iDFT()

        iDFT_Bar.Value = 0

        'THE INVERSE DISCRETE FOURIER TRANSFORM
        'copyright © 1997-1999 by California Technical Publishing
        'published with  permission from Steven W Smith, www.dspguide.com
        'The time domain signal, held in XX[ ], is calculated from the frequency domain signals,
        'held in REX[ ] and IMX[ ].

        'Find the cosine and sine wave amplitudes using Eq. 8-3

        For i As Integer = 0 To NM1D2
            REX(i) = REX(i) / NM1D2
            IMX(i) = -IMX(i) / NM1D2
        Next i

        REX(0) = REX(0) / 2
        REX(NM1D2) = REX(NM1D2) / 2

        For i As Integer = 0 To N 'Zero XX[ ] so it can be used as an accumulator
            XX(i) = 0
        Next i

        ' Eq. 8-2 SYNTHESIS METHOD #1. Loop through each
        ' frequency generating the entire length of the sine and cosine
        ' waves, and add them to the accumulator signal, XX[ ]

        For K = 0 To NM1D2 'K% loops through each sample in REX[ ] and IMX[ ]
            For i As Integer = 0 To N 'I% loops through each sample in XX[ ]
                XX(i) = XX(i) + REX(K) * Math.Cos(2 * Math.PI * K * i / N)
                XX(i) = XX(i) + IMX(K) * Math.Sin(2 * Math.PI * K * i / N)
            Next i
        Next K

        iDFT_Bar.Value = 100

    End Sub

    Public Sub do_DFT()

        iDFT_Bar.Value = 0

        'THE DISCRETE FOURIER TRANSFORM
        'copyright © 1997-1999 by California Technical Publishing
        'published with permission from Steven W Smith, www.dspguide.com
        'The frequency domain signals, held in REX[ ] and IMX[ ], are calculated from
        'the time domain signal, held in XX[ ].

        For K = 0 To NM1D2 'Zero REX[ ] & IMX[ ] so they can be used as accumulators
            REX(K) = 0
            IMX(K) = 0
        Next

        'Correlate XX[ ] with the cosine and sine waves, Eq. 8-4
        For K = 0 To NM1D2 'K% loops through each sample in REX[ ] and IMX[ ]
            For i As Integer = 0 To NM1 'I% loops through each sample in XX[ ]
                REX(K) = REX(K) + XX(i) * Math.Cos(2 * Math.PI * K * i / N)
                IMX(K) = IMX(K) - XX(i) * Math.Sin(2 * Math.PI * K * i / N)
            Next i
        Next K

        iDFT_Bar.Value = 100

    End Sub


    Private Sub FFT()

        FFT_Bar.Value = 0

        'THE FAST FOURIER TRANSFORM
        'copyright © 1997-1999 by California Technical Publishing
        'published with permission from Steven W Smith, www.dspguide.com
        'Upon entry, N% contains the number of points in the DFT, REX[ ] and
        'IMX[ ] contain the real and imaginary parts of the input. Upon return,
        'REX[ ] and IMX[ ] contain the DFT output. All signals run from 0 to N%-1.

        Dim N As Integer, NM1 As Integer, ND2 As Integer, M As Integer, J As Integer
        Dim TR As Double, TI As Double
        Dim K As Integer
        Dim LE As Integer, LE2 As Integer
        Dim UR As Double, UI As Double, SR As Double, SI As Double
        Dim JM1 As Integer
        Dim IP As Integer

        N = FFT_length

        For i As Integer = 0 To (FFT_length)
            REX(i) = FFT_in(i) 'load REX IMX buffer with data
            IMX(i) = 0
        Next

        NM1 = N - 1
        ND2 = N / 2
        M = CInt(Math.Log(N) / Math.Log(2))
        J = ND2

        For i = 1 To N - 2 'Bit reversal sorting
            If i >= J Then GoTo 2190
            TR = REX(J)
            TI = IMX(J)
            REX(J) = REX(i)
            IMX(J) = IMX(i)
            REX(i) = TR
            IMX(i) = TI
2190:       K = ND2
2200:       If K > J Then GoTo 2240
            J = J - K
            K = K / 2
            GoTo 2200
2240:       J = J + K
        Next i

        For L = 1 To M 'Loop for each IIR_stage
            LE = CInt(2 ^ L)
            LE2 = LE / 2
            UR = 1
            UI = 0
            SR = Math.Cos(Math.PI / LE2) 'Calculate sine & cosine values
            SI = -Math.Sin(Math.PI / LE2)
            For J = 1 To LE2 'Loop for each sub DFT
                JM1 = J - 1
                For i = JM1 To NM1 Step LE 'Loop for each butterfly
                    IP = i + LE2
                    TR = REX(IP) * UR - IMX(IP) * UI 'Butterfly calculation
                    TI = REX(IP) * UI + IMX(IP) * UR
                    REX(IP) = REX(i) - TR
                    IMX(IP) = IMX(i) - TI
                    REX(i) = REX(i) + TR
                    IMX(i) = IMX(i) + TI
                Next i
                TR = UR
                UR = TR * SR - UI * SI
                UI = TR * SI + UI * SR
            Next J
        Next L

        For i As Integer = 0 To (FFT_length)
            FFT_out_module(i) = Math.Sqrt((REX(i) ^ 2) + (IMX(i) ^ 2))
            If (REX(i) > 0 And IMX(i) > 0) Then FFT_out_phase(i) = (Math.Atan(IMX(i) / REX(i))) * (180 / Math.PI) '1st quadrant
            If (REX(i) <= 0 And IMX(i) > 0) Then FFT_out_phase(i) = (Math.Atan(IMX(i) / REX(i))) * (180 / Math.PI) + 180 '2nd quadrant
            If (REX(i) <= 0 And IMX(i) <= 0) Then FFT_out_phase(i) = (Math.Atan(IMX(i) / REX(i))) * (180 / Math.PI) + 180 '3rd quadrant
            If (REX(i) > 0 And IMX(i) <= 0) Then FFT_out_phase(i) = (Math.Atan(IMX(i) / REX(i))) * (180 / Math.PI) + 360 '4th quadrant
        Next

        FFT_Bar.Value = 100

    End Sub

    Public Sub Bode_plot()

        Me.GainPlot.Series.Clear()

        With Me.GainPlot
            With .Series.Add("Gain_LPF")
                For i As Integer = 1 To (FFT_length / 2) - 1
                    F = FS * (i / FFT_length)
                    .Points.AddXY(F, (Math.Log10(Signal_LPF_module(i) / Signal_module(i) + 0.000000001)) * 20) 'decibels
                Next
            End With
        End With

        Me.GainPlot.Series("Gain_LPF").BorderWidth = 2
        Me.GainPlot.Series("Gain_LPF").Color = Color.Blue
        Me.GainPlot.Series("Gain_LPF").ChartType = DataVisualization.Charting.SeriesChartType.FastLine

        With Me.GainPlot
            With .Series.Add("Gain_HPF")
                For i As Integer = 1 To (FFT_length / 2) - 1
                    F = FS * (i / FFT_length)
                    .Points.AddXY(F, (Math.Log10(Signal_HPF_module(i) / Signal_module(i) + 0.000000001)) * 20) 'decibels
                Next
            End With
        End With

        Me.GainPlot.Series("Gain_HPF").BorderWidth = 2
        Me.GainPlot.Series("Gain_HPF").Color = Color.Red
        Me.GainPlot.Series("Gain_HPF").ChartType = DataVisualization.Charting.SeriesChartType.FastLine

        With Me.GainPlot
            With .Series.Add("Gain_LPF_TARGET")
                For i As Integer = 0 To N
                    F = FS * (i / N)
                    .Points.AddXY(F, (Math.Log10(GAIN_LPF_TARGET(i) + 0.000000001)) * 20) 'decibels
                Next
            End With
        End With

        Me.GainPlot.Series("Gain_LPF_TARGET").MarkerSize = 8
        Me.GainPlot.Series("Gain_LPF_TARGET").MarkerStyle = MarkerStyle.Cross
        Me.GainPlot.Series("Gain_LPF_TARGET").Color = Color.LightBlue
        Me.GainPlot.Series("Gain_LPF_TARGET").ChartType = DataVisualization.Charting.SeriesChartType.FastPoint

    End Sub

    Private Sub Signal_generate()

        For i As Integer = 0 To (BUF_length)
            Signal_noise(i) = (Rnd() - 0.5) 'White Noise
        Next

    End Sub

    Private Sub Fmin_Box_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Fmin_Combo.SelectedIndexChanged
        GainPlot.ChartAreas(0).AxisX.Minimum = Me.Fmin_Combo.Text
    End Sub

    Private Sub Fmax_Box_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Fmax_Combo.SelectedIndexChanged
        GainPlot.ChartAreas(0).AxisX.Maximum = Me.Fmax_Combo.Text
    End Sub

    Private Sub dBmax_Bode_Box_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles dBmax_Combo.SelectedIndexChanged
        GainPlot.ChartAreas(0).AxisY.Maximum = Me.dBmax_Combo.Text
    End Sub

    Private Sub dBmin_Bode_Box_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles dBmin_Combo.SelectedIndexChanged
        GainPlot.ChartAreas(0).AxisY.Minimum = Me.dBmin_Combo.Text
    End Sub

    Private Sub Bode_Window_Box_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BodeWindow_List.SelectedIndexChanged

        Signal_generate() 'generate the source signal used for the Bode Plot (should be white noise)
        Signal_window_generate() 'generate weighting window

        x = 1
        For i As Integer = MID To (FFT_length + MID - 1) ' transfer windowed signal into FFT routine
            FFT_in(x) = Signal_window(i) * Signal_noise(i) 'multiply signal by window
            x = x + 1
        Next
        FFT() 'execute FFT routine on input signal
        For i As Integer = 1 To (FFT_length) 'save input signal module
            Signal_module(i) = FFT_out_module(i)
        Next

        For i As Integer = 0 To (BUF_length) 'transfer source signal to the FIR filter
            FIR_in(i) = Signal_noise(i)
        Next

        do_FIR() 'execute FIRLPF filter and FIRHPF on input signal

        For i As Integer = 0 To (BUF_length) 'save FIRLPF filtered signal
            Signal_LPF(i) = FIRLPF_out(i)
        Next
        x = 1
        For i As Integer = MID To (FFT_length + MID - 1) ' transfer LPF filtered windowed signal into FFT routine
            FFT_in(x) = Signal_window(i) * Signal_LPF(i)
            x = x + 1
        Next
        FFT() 'execute FFT routine on FIRLPF filtered signal
        For i As Integer = 0 To (FFT_length) 'save FIRLPF filtered signal module
            Signal_LPF_module(i) = FFT_out_module(i)
        Next

        For i As Integer = 0 To (BUF_length) 'save FIRHPF filtered signal
            Signal_HPF(i) = FIRHPF_out(i)
        Next
        x = 1
        For i As Integer = MID To (FFT_length + MID - 1) ' transfer HPF filtered windowed signal into FFT routine
            FFT_in(x) = Signal_window(i) * Signal_HPF(i)
            x = x + 1
        Next
        FFT() 'execute FFT routine on FIRHPF filtered signal
        For i As Integer = 0 To (FFT_length) 'save FIRHPF filtered signal module
            Signal_HPF_module(i) = FFT_out_module(i)
        Next

        Bode_plot()

    End Sub

    Private Sub FS_Box_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FS_List.SelectedIndexChanged
        FS = Me.FS_List.Text
        ComputeALL()
    End Sub

    Private Sub FFT_length_Box_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FFT_length_List.SelectedIndexChanged
        FFT_length = Me.FFT_length_List.Text
        ComputeALL()
    End Sub

    Private Sub FIR_length_List_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FIR_length_List.SelectedIndexChanged
        FIR_length = Me.FIR_length_List.Text
        ComputeALL()
    End Sub

    Private Sub Family_List_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Lowpass_List.SelectedIndexChanged
        ComputeALL()
    End Sub

    Private Sub Order_List_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Order_List.SelectedIndexChanged
        ComputeALL()
    End Sub

    Private Sub Double_Box_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Double_Box.CheckedChanged
        ComputeALL()
    End Sub

    Private Sub Window_List_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Window_List.SelectedIndexChanged
        ComputeALL()
    End Sub

End Class
